# Note:  This is loaded in the namespace
# variable started 0

if {![info exists started]} {variable started 0 }

if {$started != 0} {
    variable JS ""
    do_click
}

if {$started == 0} {  ;#  first time load code and define local variables
    set started 1
    
    variable JS ""
    
    proc do_redisplay {} {;}
    
    proc do_test01_fast { params } { 
       global SIDEList
       global ASICList0 ASICList1
	
	# It powers up 8 non-adjacent channels each time.
	
	variable msgIden
	
	# set name [lindex [lindex $params 1] 0]
	set sideUnderTest [lindex [lindex $params 1] 1]
	set asicUnderTest [lindex [lindex $params 1] 2]
	set vcall         [lindex [lindex $params 1] 3] ;# [V]
	set vcalh         [lindex [lindex $params 1] 4] ;# [V]
	set pulseNumber   [lindex [lindex $params 1] 5] ;# [a.u.]
	set pulseWidth    [lindex [lindex $params 1] 6] ;# [10ns]
	set pulseDelay    [lindex [lindex $params 1] 7] ;# [10ns]
	
	do_getState 
	# Start DAQ.
	after 500 set timeout 1
	do_go
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 1 } {return "Daq not started properly.";}
	
	# Loop over individual side.
	foreach itrSide [set SIDEList]  {
	    
	    if {$itrSide != $sideUnderTest} {continue;}
	    
	    # Loop over individual asic.
	    foreach itrAsic [set ASICList$itrSide]  {
		
		if {$asicUnderTest != "all" && $asicUnderTest != $itrAsic } {continue;}
		
		# Set file name
		set fileName  /MIDAS/SpecData/R3B/TestData/
		append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
		append fileName "Test01-"
		append fileName "Side$itrSide-Asic$itrAsic.txt"
		
		# Create file and set permissions.
		set flagOpen [catch {open $fileName w} fileInput]
		set flagPerm [catch {file attributes $fileName -permissions rwxrwxrwx} ]
		if { $flagOpen != 0 } { InsertLog "Cannot open $fileName." }
		if { $flagPerm != 0 } { InsertLog "Error setting permission of $fileName." }
		
		# Enable test pulse injection
		set_asicValidate $itrSide $itrAsic 1
		# Set pulse params
		do_loadPulseParams $pulseWidth $pulseDelay $pulseNumber
		
		# Set pulse amplitude.
		set voltStep [expr 2.0/256] ;# [V]
		set valueVcalh [expr round( ($vcalh)/$voltStep ) ]
		set valueVcall [expr round( ($vcall)/$voltStep ) ]
		set registerVcalh 16
		set registerVcall 17
		WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcalh] $valueVcalh
		WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcall] $valueVcall
		
		# Loop over individual channels.
		set channelNumber 128
		set channelGroup 4
		for {set itrGroup 0} {$itrGroup < $channelNumber/$channelGroup} {incr itrGroup} {
		    
		    # To run on subset of channel.
		    # if {$itrGroup >= 4 } { continue;}
		    
		    for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			
			set valuePlsInj [expr int( pow (2,  $dummyChannel - ( $dummyChannel/8)*8 ) ) ]
			set registerPlsInj [expr int ( 64 + ($dummyChannel/8) )]
		 	WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPlsInj] $valuePlsInj
			
			set valuePwr [expr 255 - int( pow (2,  $dummyChannel - ( $dummyChannel/8)*8 ) ) ]
			set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
		    }
		    after 1000
		    
		    # Inject pulses 
		    do_testPulse $pulseWidth $pulseDelay $pulseNumber
		    
		    for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			set fileName "E.$itrSide.$itrAsic.$dummyChannel"
			# Empty the content of the histogram 
			# Check also that the histogram has been created.
			if { [ZeroSpectrum $fileName] != 0 } { 
			    return "Histograms E.$itrSide.$itrAsic.* were not created";}
		    }
		    do_testPulse $pulseWidth $pulseDelay $pulseNumber
		    
		    for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			set plot  [do_analyseSpectra $itrSide $itrAsic $dummyChannel]
			set range [lindex $plot 0]
			set data  [lindex $plot 1]
			puts $fileInput "$itrSide $itrAsic $dummyChannel $range"
			puts $fileInput $data
			flush $fileInput
		    }
		    
		    # Power off channel.
		    for {set itrChannel 0} {$itrChannel < $channelGroup} {incr itrChannel} {
			set dummyChannel [expr $itrGroup + ($itrChannel* ($channelNumber/$channelGroup) ) ]
			
			set registerPwr [expr int ( 80 + ($dummyChannel/8) )]
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
			
			set registerPlsInj [expr int ( 64 + ($dummyChannel/8) )]
			WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPlsInj] 0
			
		    }
		} ;# end of loop over channel.
		
		close $fileInput
		# Disable test pulse injection
		set_asicValidate $itrSide $itrAsic 0 
		
	    } ;# end of loop over asic.
	} ;#end of loop over side.
	
	# Stop DAQ
	do_getState
	after 500 set timeout 1
	do_stop
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 0 } {return "Daq did not stop correctly";}
	
	return "Test completed (fast)"
	
    }
    
    proc do_test01_slow { params } {
       global SIDEList
       global ASICList0 ASICList1
	
	# It powers up only 1 channel each time.
	
	variable msgIden
	
	# set name [lindex [lindex $params 1] 0]
	set sideUnderTest [lindex [lindex $params 1] 1]
	set asicUnderTest [lindex [lindex $params 1] 2]
	set vcall         [lindex [lindex $params 1] 3] ;# [V]
	set vcalh         [lindex [lindex $params 1] 4] ;# [V]
	set pulseNumber   [lindex [lindex $params 1] 5] ;# [a.u.]
	set pulseWidth    [lindex [lindex $params 1] 6] ;# [10ns]
	set pulseDelay    [lindex [lindex $params 1] 7] ;# [10ns]
	
	do_getState 
	# Start DAQ.
	after 500 set timeout 1
	do_go
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 1 } {return "Daq not started properly.";}
	
	# Loop over individual side.
	foreach itrSide [set SIDEList]  {
	    
	    if {$itrSide != $sideUnderTest} {continue;}
	    
	    # Loop over individual asic.
	    foreach itrAsic [set ASICList$itrSide]  {
		
		if {$asicUnderTest != "all" && $asicUnderTest != $itrAsic } {continue;}
		
		# Set file name
		set fileName  /MIDAS/SpecData/R3B/TestData/
		append fileName [clock format [clock seconds] -format "%Y-%m-%d-"]
		append fileName "Test01-"
		append fileName "Side$itrSide-Asic$itrAsic.txt"
		
		# Create file and set permissions.
		set flagOpen [catch {open $fileName w} fileInput]
		set flagPerm [catch {file attributes $fileName -permissions rwxrwxrwx} ]
		if { $flagOpen != 0 } { InsertLog "Cannot open $fileName." }
		if { $flagPerm != 0 } { InsertLog "Error setting permission of $fileName." }
		
		# Enable test pulse injection
		set_asicValidate $itrSide $itrAsic 1
		# Set pulse params
		do_loadPulseParams $pulseWidth $pulseDelay $pulseNumber
		
		# Set pulse amplitude.
		set voltStep [expr 2.0/256] ;# [V]
		set valueVcalh [expr round( ($vcalh)/$voltStep ) ]
		set valueVcall [expr round( ($vcall)/$voltStep ) ]
		set registerVcalh 16
		set registerVcall 17
		WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcalh] $valueVcalh
		WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerVcall] $valueVcall
		
		# Loop over individual channels.
		set channelNumber 128
		for {set itrChannel 0} {$itrChannel < $channelNumber} {incr itrChannel} {
		    
		    # To run on subset of channel.
		    if {$itrChannel > 20 } { continue;}
		    
		    set valuePwr [expr 255 - int( pow (2, $itrChannel - ($itrChannel/8)*8 ) ) ]
		    set registerPwr [expr int ( 80 + ($itrChannel/8) )]
		    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] $valuePwr
		    # It seems not to be necessary: Wait for the channel to switch on properly.
		    # after 500
		    
		    # Inject pulses 
		    do_testPulse $pulseWidth $pulseDelay $pulseNumber
		    
		    set fileName "E.$itrSide.$itrAsic.$itrChannel"
		    # Empty the content of the histogram 
		    # Check also that the histogram has been created.
		    if { [ZeroSpectrum $fileName] != 0 } { 
		      	return "Histograms E.$itrSide.$itrAsic.* were not created";}
		    
		    do_testPulse $pulseWidth $pulseDelay $pulseNumber
		    
		    set plot  [do_analyseSpectra $itrSide $itrAsic $itrChannel]
		    set range [lindex $plot 0]
		    set data  [lindex $plot 1]
		    puts $fileInput "$itrSide $itrAsic $itrChannel $range"
		    puts $fileInput $data
		    flush $fileInput
		    
		    # Power off channel.
		    WriteR3BASICRegister [expr ($itrSide << 18) + ($itrAsic << 10) + $registerPwr] 255
		} ;# end of loop over channel.
		
		close $fileInput
		# Disable test pulse injection
		set_asicValidate $itrSide $itrAsic 0 
		
	    } ;# end of loop over asic.
	} ;#end of loop over side.
	
	# Stop DAQ
	do_getState
	after 500 set timeout 1
	do_stop
	vwait timeout
	# Mandatory to enquire state
	if { [do_getState] != 0 } {return "Daq did not stop correctly";}
	
	return "Test completed (slow)"
    }
    
    proc  do_click {}  {
	variable cgidata
	variable started
	variable LogFlag
	variable ShowLogFlag
	
	set cgidata [ncgi::nvlist]
	
	# InsertLog "cgidata= ([llength $cgidata]) $cgidata"
	
	if {[llength $cgidata] == 0} then {return}
	
	if {[llength $cgidata] < 2} then {
	    InsertLog "CGI data is too short: $cgidata"
	} else {
	    if {[lindex $cgidata 0] != "Widget"} then {
		InsertLog "Invalid CGI data: $cgidata"
	    } else {
		
		set w [first [lindex $cgidata 1]]
		
		switch -glob $w {
		    
		    INITIALIZE_ASICS {  InsertLog "[do_powerOffFrontEnd]"; InsertLog "[do_enableCalibration]"}
		    INITIALIZE_DAQ   { InsertLog "[do_initializeDaq $cgidata]" }
		    RESET    { set started 0; }
		    TEST01_SLOW { InsertLog "[do_test01_slow $cgidata]" }
		    TEST01_FAST { InsertLog "[do_test01_fast $cgidata]" }
		    
		    default {InsertLog "unexpected CGI data:  $cgidata"}
		}
	    }
        }
	do_redisplay
    }
    
}   ;# end of if {$started == 0}

