
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include <unistd.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <sys/errno.h>

#include <time.h>

#include <linux/i2c.h>
#include <linux/i2c-dev.h>

void Abort() {exit(16);}



void Usage(char *progname) {
    printf("Usage\n%s -a [i2c address 0=>15 ] -o [offset 0=>255] -l [loops]\n",progname);

    exit(1);
}

        
int main(int argc, char **argv) {

    unsigned int ADDR = 16;
    unsigned int OFFSET = 0x0;
    unsigned int LOOPS = 1;

    int DelayTicks=0;     /*  millisecs  */

    int fh = -1;
    int rc;
    int i;
    int loop;

    unsigned char Data8;

    struct timespec time_request;

    if (argc >1) {
        for(i=1;i <argc;i++) {
            if ( (argv[i][0] == '-') || (argv[i][0] == '/') ) {
                switch(tolower(argv[i][1])) {
                    case 'a':
                        ADDR = atoi(argv[++i]);
                        break;
                    case 'o':
                        OFFSET = atoi(argv[++i]);
                        break;
                   case 'l':
                        LOOPS = atoi(argv[++i]);
                        break;
                    case 'w':
                        DelayTicks = atoi(argv[++i]);
                        break;
                    default:
                        Usage(argv[0]);
                        break;
                }
            }
            else
                Usage(argv[0]);
        }
    }

    //  connect to i2c device

    fh = open("/dev/i2c/0", O_RDWR);
    if (fh < 0) {printf("open failed for i2c device\n"); Abort();}

    printf("open OK for i2c device\n");

    //   select Slave device

    rc = ioctl(fh, I2C_SLAVE_FORCE, ADDR);
    if (rc < 0) {printf("ioctl failed for address %d\n", ADDR); Abort();}

    printf("ioctl OK for address %d\n", ADDR);

    for (loop=0; loop<LOOPS; loop++)
    {

    //    define address offset

    Data8 = (unsigned char) (OFFSET & 255);   /* offset address is 8 bit */
    rc = write(fh, &Data8, 1);
    if (rc != 1) {printf("write failed for offset %d rc=%d\n", Data8, rc); Abort();}

    printf("write OK for offset %d\n", Data8);

    //    read data

    rc = read(fh, &Data8, 1);
    if (rc != 1) {printf("read failed rc=%d\n",rc); Abort();}

    printf("read OK\n");

    printf("ADDR %d; OFFSET %d returns 0x%x\n", ADDR, OFFSET, Data8);

// if (Data8 == 1) {printf("Beware: data returned is 1 - this could be a bus timeout - read register having value ne 1!\n");}

    if (DelayTicks != 0) {

       time_request.tv_sec = DelayTicks/1000;
       time_request.tv_nsec = (DelayTicks - (time_request.tv_sec * 1000)) * 1000000;
       i = nanosleep(&time_request, NULL);

    }

    }

    (void) close(fh);

    exit(0);
}






