

#    experiment run control for R3B

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

global Options

global NumASICs NumChans
set NumASICs 32
set NumChans 128


variable SpecBase [file join $env(MIDASBASE) SpecData R3B]
variable SpecPath

proc Dump_Raw_Data {} {return "[Dump_Raw_R3B_Data]\n"}

proc Set_Options {} {
    global Options

    LoadOptions
    
    if {![info exists Options(R3B_GroupBase)]} {set Options(R3B_GroupBase) 1}
    if {![info exists Options(Include.R3B)]}   {set Options(Include.R3B) 1}

    if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 8192}
    if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 8192}
    if {![info exists Options(Stat.shift)]}    {set Options(Stat.shift) 0}
    if {![info exists Options(Stat.offset)]}   {set Options(Stat.offset) 0}

    if {![info exists Options(R3B.channels)]} {set Options(R3B.channels) 4096}
    if {![info exists Options(R3B.shift)]}    {set Options(R3B.shift) 0}
    if {![info exists Options(R3B.offset)]}   {set Options(R3B.offset) 0}

    if {![info exists Options(R3B.Tchannels)]} {set Options(R3B.Tchannels) 40000}
    if {![info exists Options(R3B.Tshift)]}    {set Options(R3B.Tshift) 0}
    if {![info exists Options(R3B.Toffset)]}   {set Options(R3B.Toffset) 0}
    
    if {![info exists Options(R3B_Hist_E_Enable)]} {set Options(R3B_Hist_E_Enable) 1}
    if {![info exists Options(R3B_Hist_N_Enable)]} {set Options(R3B_Hist_N_Enable) 0}
    if {![info exists Options(R3B_Hist_T_Enable)]} {set Options(R3B_Hist_T_Enable) 0}

    SaveOptions

}

proc Dump_Raw_R3B_Data {} {
    
    set info "data $::DataAcquisition::ACQSERVER\n"
    
    for {set offset 0} {$offset <= 500} {incr offset 100} {
	
	DataAcqCommand 19 "Getting Raw Data" "19 $offset" 2
	set data [ReadNetVar NetVar.EXEC.Response]
	
	set c 0
	
	for {set i 0} {$i < [llength $data]} {incr i} {
	    set word [lindex $data $i]
	    if {$word == 0xaa5555aa} then {
		append info "\n$word - end of dump\n"
		break
	    }
	    if {$word == 0x55aaaa55} then {
		append info "\n$word - end of event\n"
		continue
	    }
	    if {[expr ($word >> 30) & 3] == 3} {
		incr i
		set t [lindex $data $i]
		set hit [expr ($word >> 29) & 1]
		set chan [expr ($word >> 12) & 127]
		set asic [expr ($word >> 19) & 15]
		set id [expr ($word >> 23) & 0x3f]
		set adc [format "0x%x" [expr $word & 0xfff]]
		append info "\nData Item $word $t; Hit=$hit; ID=$id; ASIC=$asic; Channel=$chan; ADC=$adc; Time=$t"
	    }
	    
	    if {[expr ($word >> 30) & 3] == 2} {
		incr i
		set t [lindex $data $i]
		set time [format "0x%x" [expr $word & 0x000fffff]]
		append info "\nTime Item $word $t; Time (high)=$time; Time (low)=$t"
	    }
	    
	    incr c
	    
	    if {$c == 10} then {append info "\n"; set c 0}
	}
    }
    
    return "$info"
}

proc create_R3B_spectra {} {
    global Options
    global NumASICs NumChans
    
    set Me $::DataAcquisition::ACQSERVER
    
    for {set Asic 0} {$Asic < $NumASICs} {incr Asic} {
	
	puts "Creating spectra for ASIC $Asic"
	
	#    creating spectra for ADC data 
	
	for {set Ch 0} {$Ch < $NumChans} {incr Ch} {
	    
	    if {$Options(R3B_Hist_E_Enable) == 1} {
		
		set z [catch {CreateSpectrum1D E.$Asic.$Ch $Options(R3B.channels)} m]
		
		#    Allow "Spectrum already exists" so that further spectra can be created
		if {$z!=0 && $m==0x30004} then {set z 0}
		if {$z!=0} then {
		    puts "Error attempting to create spectrum E.$Asic.$Ch"
		    error $m
		}
		
		set z [catch {SpectrumPutTitle E.$Asic.$Ch [set Me]_E.$Asic.$Ch} m]
		
	    }
	    
	    if {$Options(R3B_Hist_N_Enable) == 1} {
		
		set z [catch {CreateSpectrum1D N.$Asic.$Ch $Options(R3B.channels)} m]
		
		#    Allow "Spectrum already exists" so that further spectra can be created
		if {$z!=0 && $m==0x30004} then {set z 0}
		if {$z!=0} then {
		    puts "Error attempting to create spectrum N.$Asic.$Ch"
		    error $m
		}
		
		set z [catch {SpectrumPutTitle N.$Asic.$Ch [set Me]_N.$Asic.$Ch} m]
		
	    }
	    
	    
	}
	
	#    Creating spectra for Time diagnostic
	
	for {set Ch 0} {$Ch < $NumChans} {incr Ch} {
	    
	    if {$Options(R3B_Hist_T_Enable) == 1} {
		
		set z [catch {CreateSpectrum1D T.$Asic.$Ch $Options(R3B.Tchannels)} m]
		
		#    Allow "Spectrum already exists" so that further spectra can be created
		if {$z!=0 && $m==0x30004} then {set z 0}
		if {$z!=0} then {
		    puts "Error attempting to create spectrum T.$Asic.$Ch"
		    error $m
		}
		
		set z [catch {SpectrumPutTitle T.$Asic.$Ch [set Me]_T.$Asic.$Ch} m]
		
	    }
	}

    }
    
}

proc map_R3B_spectra {} {
    upvar #0 SpectraHandle f
    global Options
    global NumASICs NumChans
    
    set Me $::DataAcquisition::ACQSERVER
    
    set SpectraDefs [file join /tmp SpectraDefs.dat]
    
    set z [catch {open $SpectraDefs w} f]
    if {$z != 0} then {
	puts "Cannot open file $SpectraDefs $f"
	return
    }
    
    map_spectrum "Stat 0 0 $Options(Stat.offset) $Options(Stat.shift)" 
    
    for {set Asic 0} {$Asic < $NumASICs} {incr Asic} {
	
	puts "Mapping spectra for ASIC $Asic"
	
	for {set Ch 0} {$Ch < $NumChans} {incr Ch} {
	    
	    if {$Options(R3B_Hist_E_Enable) == 1} {
		
		set group [expr $Options(R3B_GroupBase)+$Asic]
		set item $Ch
		map_spectrum "E.$Asic.$Ch $group $item $Options(R3B.offset) $Options(R3B.shift)"
	    }
	    if {$Options(R3B_Hist_N_Enable) == 1} {
		
		set group [expr $Options(R3B_GroupBase)+$Asic+16]
		set item $Ch
		map_spectrum "N.$Asic.$Ch $group $item $Options(R3B.offset) $Options(R3B.shift)"
	    }
	}
	
	for {set Ch 0} {$Ch < $NumChans} {incr Ch} {
	    
	    if {$Options(R3B_Hist_T_Enable) == 1} {
		
		set group [expr $Options(R3B_GroupBase)+$Asic+32]
		set item $Ch
		map_spectrum "T.$Asic.$Ch $group $item $Options(R3B.Toffset) $Options(R3B.Tshift)"
	    }
	}
	
    }
    
    close $f
}


proc doSetUp {} {
    global Options
    
    puts "executing doSetUp"
    
    Set_Options; make-UIregisters
    
    DataAcqCommand 1 "Halt"

    unmap_spectra
    
    delete_spectra
    
    set Me $::DataAcquisition::ACQSERVER
    
    CreateSpectrum1D Stat $Options(Stat.channels)
    set z [catch {SpectrumPutTitle Stat [set Me]_Stat} m]
    
    CreateSpectrum1D Rate $Options(Rate.channels)
    set z [catch {SpectrumPutTitle Rate [set Me]_Rate} m]
    
    create_R3B_spectra

    map_R3B_spectra 
    
    map_spectra

    DataAcqCommand 3 "Setup"

    WriteNetVar NetVar.DACQ.PollTicks 100
    WriteNetVar NetVar.DACQ.Crdoutenable 1   ;# enable data readout
    
    puts "completed doSetUp"
}

proc TerminateExec {} {

    unmap_spectra
    DataAcqCommand 22 "Terminating DataAcq program" 0 2

}

proc LoadExec {} {
    global env
    
    set z [catch {eval exec nice [file join $env(MIDASBASE) Data_Acq bin Linux64 R3BExecV2] &} m]
    if {$z != 0} {puts "RunControl LoadExec returned z=$z and $m"}
    
}

make-UIregisters

Set_Options


