#    This is a simple terminal emulation program (like Hyperterminal or Teraterm)

set frame  terminal
set title "Terminal"
set icon  terminal

set started 0

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set PortList {null}    ;#  silly default
set Port [lindex $PortList 0]

   switch $env(platform) {
     windows  {
         set PortList {COM1 COM2 COM3 COM4 COM5 COM6 COM7 COM8 COM9 COM10}
         set Port [lindex $PortList 0]
     }
     unix {
        switch $env(os) {
           SunOS    {
               set PortList {/dev/ttya /dev/ttyb /dev/ttyc}
               set Port [lindex $PortList 0]
           }
           Linux64 -
           Linux  {
               set PortList {/dev/ttyS0 /dev/ttyS1 /dev/ttyUSB0 /dev/ttyUSB1 /dev/ttyUSB2 /dev/ttyUSB3}
               set Port [lindex $PortList 2]
           }
           default  {puts stderr "$env(os) not supported"}
        }
     }
     default {puts stderr "$env(platform) not supported"}
   }


set BaudList {9600 600 1200 2400 4800 9600 14400 19200 38400 57600 115200}
set Baud [lindex $BaudList 0]

set DataList {8 7 6 5}
set Data [lindex $DataList 0]

set ParityList {none odd even mark space}
set Parity [lindex $ParityList 0]

set StopList {1 2}
set Stop [lindex $StopList 0]

set FlowList {none rtscts xonxoff}
set Flow [lindex $FlowList 0]

set Cmd ""

set channelId 0

proc make-frame {} {
   global env PortList BaudList DataList ParityList StopList FlowList

set rows 24
set cols 80

  :frame [self frame-name] :label "[self frame-title]" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10
    :choice-stack PORT   :label "Port" :strings $PortList :value 2
    :choice-stack BAUD   :label "Baud rate" :strings $BaudList
    :choice-stack DATA   :label "Data bits" :strings $DataList 
    :choice-stack PARITY :label "Parity" :strings $ParityList
    :choice-stack STOP   :label "Stop bits" :strings $StopList
    :choice-stack FLOW   :label "Flow" :strings $FlowList

    :next-row T
    :button FINISH     :label Finish     :command do_finish
    :hskip 100
    :button CONNECT    :label Connect    :w 12 :command do_connect
    :button DISCONNECT :label Disconnect :w 12 :command do_disconnect
    :hskip 100
    :button CLEAR   :label "Clear log window" :command ":modify LOG :empty T"

  :panel P1 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font fixed
     configuretags

  :panel P3 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10
    :text COMMAND :label "" :w $cols 

   open-frame

  :galley_size LOG $rows $cols

    focus [:path COMMAND].entry

   :show T
}

proc click-configure {W} {:galley_resize LOG}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc insert {s} {:modify LOG :usingtag taginsert :strings $s\n}

proc start args {
   global started

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
}

proc click arg {
   global env PortList BaudList DataList ParityList StopList FlowList
   global Port Baud Data Parity Stop Flow
   global Cmd

   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer
   switch -glob $w  {

      PORT  {set Port [lindex $PortList $v]}
      BAUD  {set Baud [lindex $BaudList $v]}
      DATA  {set Data [lindex $DataList $v]}
      PARITY  {set Parity [lindex $ParityList $v]}
      STOP  {set Stop [lindex $StopList $v]}
      FLOW  {set Flow [lindex $FlowList $v]}

      COMMAND {set Cmd $v; do_command}

      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }
}

proc do_finish {} {
    global channelId 

   if {$channelId != 0} {do_disconnect}
   DeleteTask
}

proc do_command {} {
    global channelId Cmd

#   :modify LOG :usingtag taginput :strings "$Cmd\n"

   if {$channelId == 0} {insert "Not connected"; return}

   puts $channelId "$Cmd"
   :modify COMMAND :value ""
}

proc do_connect {} {
   global env
   global channelId Port
   global Port Baud Data Parity Stop Flow

   if {$channelId != 0} {
       insert "Already connected: Disconnect first"
   } else {

       if {$env(os) == "Linux" || $env(os) == "Linux64"} {
           set z [catch {exec sudo -n chmod 666 $Port} m]   ;# need rw access to serial port
           if {$z != 0} {insert "failed to change access mode to I/O port\n$m"}
       }

       set z [catch {open $Port RDWR} m]
       if {$z != 0} {insert "failed to open access to I/O port\n$m"} else {set channelId $m}
       switch $Parity {
           none   {set parity n}
           odd    {set parity o}
           even   {set parity e}
           mark   {set parity m}
           space  {set parity s}
       }

       fconfigure $channelId -mode $Baud,$parity,$Data,$Stop -handshake $Flow
       fconfigure $channelId -blocking no -buffering none -translation binary
       fileevent $channelId readable GetData
       insert "Connected to channel $channelId"
   }
}

proc do_disconnect {} {
   global channelId Port

   if {$channelId == 0} {
       insert "Already disconnected"
   } else {
       close $channelId
       insert "Channel $channelId disconnected"
       set channelId 0
   }
}

proc GetData {args} {
    global channelId
    set data [read $channelId 132]
    set line ""
    for {set i 0} {$i < [string length $data]} {incr i} {
        set c [string range $data $i $i]
        if {$c == "\0"} {continue}
        if {$c == "\r"} {continue}
        if {$c == "\f"} {continue}
        append line $c
    }
#    if {[string length $line] == 0} {insert "null line received"; do_disconnect; after 1000; do_connect}
    :modify LOG :usingtag tagoutput :strings "$line"
}
